# !/usr/bin/env python3

from hdwallet.symbols import *


def test_symbols():

    assert ANON == "ANON"
    assert AGM == "AGM"
    assert XAX == "XAX"
    assert AYA == "AYA"
    assert AC == "AC"
    assert ATOM == "ATOM"
    assert AUR == "AUR"
    assert AXE == "AXE"
    assert BTA == "BTA"
    assert BEET == "BEET"
    assert BELA == "BELA"
    assert BTDX == "BTDX"
    assert BSD == "BSD"
    assert BCH == "BCH"
    assert BTG == "BTG"
    assert BTC == "BTC"
    assert BTCTEST == "BTCTEST"
    assert XBC == "XBC"
    assert BSV == "BSV"
    assert BTCZ == "BTCZ"
    assert BTX == "BTX"
    assert BLK == "BLK"
    assert BST == "BST"
    assert BND == "BND"
    assert BNDTEST == "BNDTEST"
    assert BOLI == "BOLI"
    assert BRIT == "BRIT"
    assert CPU == "CPU"
    assert CDN == "CDN"
    assert CCN == "CCN"
    assert CLAM == "CLAM"
    assert CLUB == "CLUB"
    assert CMP == "CMP"
    assert CRP == "CRP"
    assert CRAVE == "CRAVE"
    assert DASH == "DASH"
    assert DASHTEST == "DASHTEST"
    assert ONION == "ONION"
    assert DFC == "DFC"
    assert DNR == "DNR"
    assert DMD == "DMD"
    assert DGB == "DGB"
    assert DGC == "DGC"
    assert DOGE == "DOGE"
    assert DOGETEST == "DOGETEST"
    assert EDRC == "EDRC"
    assert ECN == "ECN"
    assert EMC2 == "EMC2"
    assert ELA == "ELA"
    assert NRG == "NRG"
    assert ETH == "ETH"
    assert ERC == "ERC"
    assert EXCL == "EXCL"
    assert FIX == "FIX"
    assert FIXTEST == "FIXTEST"
    assert FTC == "FTC"
    assert FRST == "FRST"
    assert FLASH == "FLASH"
    assert FJC == "FJC"
    assert GCR == "GCR"
    assert GAME == "GAME"
    assert GBX == "GBX"
    assert GRC == "GRC"
    assert GRS == "GRS"
    assert GRSTEST == "GRSTEST"
    assert NLG == "NLG"
    assert HNC == "HNC"
    assert THC == "THC"
    assert HUSH == "HUSH"
    assert IXC == "IXC"
    assert INSN == "INSN"
    assert IOP == "IOP"
    assert JBS == "JBS"
    assert KOBO == "KOBO"
    assert KMD == "KMD"
    assert LBC == "LBC"
    assert LINX == "LINX"
    assert LCC == "LCC"
    assert LTC == "LTC"
    assert LTCTEST == "LTCTEST"
    assert LTZ == "LTZ"
    assert LKR == "LKR"
    assert LYNX == "LYNX"
    assert MZC == "MZC"
    assert MEC == "MEC"
    assert MNX == "MNX"
    assert MONA == "MONA"
    assert MONK == "MONK"
    assert XMY == "XMY"
    assert NIX == "NIX"
    assert NMC == "NMC"
    assert NAV == "NAV"
    assert NEBL == "NEBL"
    assert NEOS == "NEOS"
    assert NRO == "NRO"
    assert NYC == "NYC"
    assert NVC == "NVC"
    assert NBT == "NBT"
    assert NSR == "NSR"
    assert OK == "OK"
    assert OMNI == "OMNI"
    assert OMNITEST == "OMNITEST"
    assert ONX == "ONX"
    assert PPC == "PPC"
    assert PSB == "PSB"
    assert PHR == "PHR"
    assert PINK == "PINK"
    assert PIVX == "PIVX"
    assert PIVXTEST == "PIVXTEST"
    assert POSW == "POSW"
    assert POT == "POT"
    assert PRJ == "PRJ"
    assert PUT == "PUT"
    assert QTUM == "QTUM"
    assert QTUMTEST == "QTUMTEST"
    assert RBTC == "RBTC"
    assert RBTCTEST == "RBTCTEST"
    assert RPD == "RPD"
    assert RVN == "RVN"
    assert RDD == "RDD"
    assert RBY == "RBY"
    assert SAFE == "SAFE"
    assert SLS == "SLS"
    assert SCRIBE == "SCRIBE"
    assert SDC == "SDC"
    assert SDCTEST == "SDCTEST"
    assert SLM == "SLM"
    assert SLMTEST == "SLMTEST"
    assert SMLY == "SMLY"
    assert SLR == "SLR"
    assert STASH == "STASH"
    assert STRAT == "STRAT"
    assert STRATTEST == "STRATTEST"
    assert SUGAR == "SUGAR"
    assert SUGARTEST == "SUGARTEST"
    assert SYS == "SYS"
    assert TOA == "TOA"
    assert THT == "THT"
    assert TRX == "TRX"
    assert TWINS == "TWINS"
    assert TWINSTEST == "TWINSTEST"
    assert USC == "USC"
    assert UNO == "UNO"
    assert VASH == "VASH"
    assert VC == "VC"
    assert XVG == "XVG"
    assert VTC == "VTC"
    assert VIA == "VIA"
    assert VIATEST == "VIATEST"
    assert VIVO == "VIVO"
    assert XWC == "XWC"
    assert WC == "WC"
    assert XUEZ == "XUEZ"
    assert XDC == "XDC"
    assert ZCL == "ZCL"
    assert ZEC == "ZEC"
    assert ZECTEST == "ZECTEST"
    assert ZEN == "ZEN"



#!/usr/bin/env python

from click_aliases import ClickAliasedGroup
from typing import Optional

from hdwallet import __version__
from hdwallet.cli.generate.hdwallet import generate_hdwallet
from hdwallet.cli.generate.addresses import generate_addresses
from hdwallet.cli.list.cryptocurrencies import list_cryptocurrencies
from hdwallet.cli.list.languages import list_languages
from hdwallet.cli.list.strengths import list_strengths
from hdwallet.cli import click


CONTEXT_SETTINGS = dict(
    help_option_names=["-h", "--help"],
)


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(__version__)
    ctx.exit()


@click.group(cls=ClickAliasedGroup,
             options_metavar="[OPTIONS]", context_settings=CONTEXT_SETTINGS)
@click.option("-v", "--version", is_flag=True, callback=print_version,
              expose_value=False, help="Show HDWallet version and exit.")
def main():
    pass


@main.group("generate", aliases=["g"], cls=ClickAliasedGroup, options_metavar="[OPTIONS]",
            short_help="Select Generate for HDWallet.", invoke_without_command=True)
@click.option("-s", "--symbol", type=str, default="BTC",
              help="Set Cryptocurrency ticker symbol.")
@click.option("-sg", "--strength", type=int, default=128,
              help="Set Strength for entropy, choose strength 128, 160, 192, 224 or 256 only.", show_default=True)
@click.option("-e", "--entropy", type=str, default=None,
              help="Set Master key from entropy hex string.", show_default=True)
@click.option("-m", "--mnemonic", type=str, default=None,
              help="Set Master key from mnemonic words.", show_default=True)
@click.option("-l", "--language", type=str, default="english",
              help="Set Language for mnemonic, choose language english, french, italian, spanish, "
                   "chinese_simplified, chinese_traditional, japanese or korean only.", show_default=True)
@click.option("-pa", "--passphrase", type=str, default=None,
              help="Set Passphrase for mnemonic.", show_default=True)
@click.option("-sd", "--seed", type=str, default=None,
              help="Set Master key from seed hex string.", show_default=True)
@click.option("-xprv", "--xprivate-key", type=str, default=None,
              help="Set Master key from xprivate key.", show_default=True)
@click.option("-xpub", "--xpublic-key", type=str, default=None,
              help="Set Master key from xpublic key.", show_default=True)
@click.option("-st", "--strict", type=bool, default=False,
              help="Set Strict for root keys.", show_default=True)
@click.option("-ac", "--account", type=int, default=0,
              help="Set derivation from account.", show_default=True)
@click.option("-ch", "--change", type=bool, default=False,
              help="Set Derivation from change.", show_default=True)
@click.option("-ad", "--address", type=int, default=0,
              help="Set Derivation from address.", show_default=True)
@click.option("-p", "--path", type=str, default=None,
              help="Set Master key derivation path.", show_default=True)
@click.option("-prv", "--private-key", type=str, default=None,
              help="Set Master key from private key.", show_default=True)
@click.option("-pub", "--public-key", type=str, default=None,
              help="Set Master key from public key.", show_default=True)
@click.option("-w", "--wif", type=str, default=None,
              help="Set Master key from wallet important format.", show_default=True)
@click.option("-sm", "--semantic", type=str, default="p2pkh",
              help="Set Semantic for xprivate and xpublic keys.", show_default=True)
@click.pass_context
def generate(
        context: click.core.Context,
        symbol: str,
        strength: int,
        entropy: Optional[str],
        mnemonic: Optional[str],
        language: Optional[str],
        passphrase: Optional[str],
        seed: Optional[str],
        xprivate_key: Optional[str],
        xpublic_key: Optional[str],
        strict: bool,
        account: int,
        change: bool,
        address: int,
        path: Optional[str],
        private_key: Optional[str],
        public_key: Optional[str],
        wif: Optional[str],
        semantic: str
):
    if context.invoked_subcommand is None:
        return generate_hdwallet(
            symbol=symbol,
            strength=strength,
            entropy=entropy,
            mnemonic=mnemonic,
            language=language,
            passphrase=passphrase,
            seed=seed,
            xprivate_key=xprivate_key,
            xpublic_key=xpublic_key,
            strict=strict,
            account=account,
            change=change,
            address=address,
            path=path,
            private_key=private_key,
            public_key=public_key,
            wif=wif,
            semantic=semantic
        )


@generate.command("addresses", aliases=["a"], options_metavar="[OPTIONS]",
                  short_help="Select Addresses for generation HDWallet addresses.")
@click.option("-s", "--symbol", type=str, default="BTC",
              help="Set Cryptocurrency ticker symbol.")
@click.option("-sg", "--strength", type=int, default=128,
              help="Set Strength for entropy, choose strength 128, 160, 192, 224 or 256 only.", show_default=True)
@click.option("-e", "--entropy", type=str, default=None,
              help="Set Master key from entropy hex string.", show_default=True)
@click.option("-m", "--mnemonic", type=str, default=None,
              help="Set Master key from mnemonic words.", show_default=True)
@click.option("-l", "--language", type=str, default="english",
              help="Set Language for mnemonic, choose language english, french, italian, spanish, "
                   "chinese_simplified, chinese_traditional, japanese or korean only.", show_default=True)
@click.option("-pa", "--passphrase", type=str, default=None,
              help="Set Passphrase for mnemonic.", show_default=True)
@click.option("-sd", "--seed", type=str, default=None,
              help="Set Master key from seed hex string.", show_default=True)
@click.option("-xprv", "--xprivate-key", type=str, default=None,
              help="Set Master key from xprivate key.", show_default=True)
@click.option("-xpub", "--xpublic-key", type=str, default=None,
              help="Set Master key from xpublic key.", show_default=True)
@click.option("-st", "--strict", type=bool, default=False,
              help="Set Strict for root keys.", show_default=True)
@click.option("-ac", "--account", type=int, default=0,
              help="Set derivation from account.", show_default=True)
@click.option("-ch", "--change", type=bool, default=False,
              help="Set Derivation from change.", show_default=True)
@click.option("-p", "--path", type=str, default=None,
              help="Set Master key derivation path.", show_default=True)
@click.option("-se", "--semantic", type=str, default="p2pkh",
              help="Set Semantic for xprivate and xpublic keys.", show_default=True)
@click.option("-h", "--hardened", type=bool, default=False,
              help="Set Hardened for addresses.", show_default=True)
@click.option("-si", "--start-index", type=int, default=0,
              help="Set Start from address index.", show_default=True)
@click.option("-ei", "--end-index", type=int, default=20,
              help="Set End to address index.", show_default=True)
@click.option("-sh", "--show", type=str, default="path,addresses:p2pkh,public_key,wif",
              help="Set Value key of generated HDWallet data to show.", show_default=True)
def addresses(
        symbol: str,
        strength: int,
        entropy: Optional[str],
        mnemonic: Optional[str],
        language: Optional[str],
        passphrase: Optional[str],
        seed: Optional[str],
        xprivate_key: Optional[str],
        xpublic_key: Optional[str],
        strict: bool,
        account: int,
        change: bool,
        path: Optional[str],
        semantic: str,
        start_index: int,
        end_index: int,
        hardened: bool,
        show: str
):
    return generate_addresses(
        symbol=symbol,
        strength=strength,
        entropy=entropy,
        mnemonic=mnemonic,
        language=language,
        passphrase=passphrase,
        seed=seed,
        xprivate_key=xprivate_key,
        xpublic_key=xpublic_key,
        strict=strict,
        account=account,
        change=change,
        path=path,
        semantic=semantic,
        start_index=start_index,
        end_index=end_index,
        hardened=hardened,
        show=show
    )


@main.group("list", aliases=["l"], cls=ClickAliasedGroup, options_metavar="[OPTIONS]",
            short_help="Select List for HDWallet information.", invoke_without_command=True)
def list():
    pass


@list.command("cryptocurrencies", aliases=["c"], options_metavar="[OPTIONS]",
              short_help="List Available cryptocurrencies of HDWallet.")
def cryptocurrencies():
    return list_cryptocurrencies()


@list.command("languages", aliases=["l"], options_metavar="[OPTIONS]",
              short_help="List Languages of mnemonic words.")
def languages():
    return list_languages()


@list.command("strengths", aliases=["s"], options_metavar="[OPTIONS]",
              short_help="List Strengths of mnemonic words.")
def strengths():
    return list_strengths()



#!/usr/bin/env python3

from setuptools import (
    setup, find_packages
)

# Project URLs
project_urls = {
    "Tracker": "https://github.com/meherett/python-hdwallet/issues",
    "Documentation": "https://hdwallet.readthedocs.io"
}

# README.md
with open("README.md", "r", encoding="utf-8") as readme:
    long_description: str = readme.read()

# requirements.txt
with open("requirements.txt", "r") as _requirements:
    requirements: list = list(map(str.strip, _requirements.read().split("\n")))

setup(
    name="hdwallet",
    version="v2.1.1",
    description="Python-based library for the implementation of a hierarchical deterministic wallet "
                "generator for more than 140+ multiple cryptocurrencies.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="ISCL",
    author="Meheret Tesfaye Batu",
    author_email="meherett@zoho.com",
    url="https://github.com/meherett/python-hdwallet",
    project_urls=project_urls,
    keywords=[
        "cryptography", "cli", "wallet", "bip32", "bip44", "bip39", "hdwallet", "cryptocurrencies", "bitcoin", "ethereum"
    ],
    entry_points={
        "console_scripts": ["hdwallet=hdwallet.cli.__main__:main"]
    },
    python_requires=">=3.6,<4",
    packages=find_packages(),
    install_requires=requirements,
    extras_require={
        "cli": [
            "click>=8.0.3,<9",
            "click-aliases>=1.0.1,<2",
            "tabulate>=0.8.9,<1"
        ],
        "tests": [
            "pytest>=6.2.5,<7",
            "pytest-cov>=3.0.0,<4"
        ],
        "docs": [
            "sphinx>=4.4.0,<5",
            "furo==2022.2.14.1",
            "sphinx-click>=3.1.0,<4"
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: ISC License (ISCL)",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ]
)



#!/usr/bin/env python3

from mnemonic import Mnemonic
from binascii import hexlify, unhexlify
from random import choice
from typing import AnyStr, Optional

import string
import os
import inspect
import unicodedata
import binascii

from hdwallet import cryptocurrencies
from .cryptocurrencies import (
    get_cryptocurrency, Cryptocurrency
)
from .libs.base58 import check_decode

# Alphabet and digits.
letters = string.ascii_letters + string.digits


def _unhexlify(integer: int):
    try:
        return unhexlify("0%x" % integer)
    except binascii.Error:
        return unhexlify("%x" % integer)


def get_semantic(_cryptocurrency: Cryptocurrency, version: bytes, key_type: str) -> str:
    for name, cryptocurrency in inspect.getmembers(cryptocurrencies):
        if inspect.isclass(cryptocurrency):
            if issubclass(cryptocurrency, cryptocurrencies.Cryptocurrency) and cryptocurrency == _cryptocurrency:
                if key_type == "private_key":
                    for key, value in inspect.getmembers(cryptocurrency.EXTENDED_PRIVATE_KEY):
                        if value == int(version.hex(), 16):
                            return key.lower()
                elif key_type == "public_key":
                    for key, value in inspect.getmembers(cryptocurrency.EXTENDED_PUBLIC_KEY):
                        if value == int(version.hex(), 16):
                            return key.lower()


def get_bytes(string: AnyStr) -> bytes:
    if isinstance(string, bytes):
        byte = string
    elif isinstance(string, str):
        byte = bytes.fromhex(string)
    else:
        raise TypeError("Agreement must be either 'bytes' or 'string'!")
    return byte


def generate_passphrase(length: int = 32) -> str:
    """
    Generate entropy hex string.

    :param length: Passphrase length, default to 32.
    :type length: int

    :returns: str -- Passphrase hex string.

    >>> from hdwallet.utils import generate_passphrase
    >>> generate_passphrase(length=32)
    "N39rPfa3QvF2Tm2nPyoBpXNiBFXJywTz"
    """

    return str().join(choice(letters) for _ in range(length))


def generate_entropy(strength: int = 128) -> str:
    """
    Generate entropy hex string.

    :param strength: Entropy strength, default to 128.
    :type strength: int

    :returns: str -- Entropy hex string.

    >>> from hdwallet.utils import generate_entropy
    >>> generate_entropy(strength=128)
    "ee535b143b0d9d1f87546f9df0d06b1a"
    """

    if strength not in [128, 160, 192, 224, 256]:
        raise ValueError(
            "Strength should be one of the following "
            "[128, 160, 192, 224, 256], but it is not (%d)."
            % strength
        )
    return hexlify(os.urandom(strength // 8)).decode()


def generate_mnemonic(language: str = "english", strength: int = 128) -> str:
    """
    Generate mnemonic words.

    :param language: Mnemonic language, default to english.
    :type language: str
    :param strength: Entropy strength, default to 128.
    :type strength: int

    :returns: str -- Mnemonic words.

    >>> from hdwallet.utils import generate_mnemonic
    >>> generate_mnemonic(language="french")
    "sceptre capter séquence girafe absolu relatif fleur zoologie muscle sirop saboter parure"
    """

    if language and language not in ["english", "french", "italian", "japanese",
                                     "chinese_simplified", "chinese_traditional", "korean", "spanish"]:
        raise ValueError("invalid language, use only this options english, french, "
                         "italian, spanish, chinese_simplified, chinese_traditional, japanese or korean languages.")
    if strength not in [128, 160, 192, 224, 256]:
        raise ValueError(
            "Strength should be one of the following "
            "[128, 160, 192, 224, 256], but it is not (%d)."
            % strength
        )

    return Mnemonic(language=language).generate(strength=strength)


def is_entropy(entropy: str) -> bool:
    """
    Check entropy hex string.

    :param entropy: Mnemonic words.
    :type entropy: str

    :returns: bool -- Entropy valid/invalid.

    >>> from hdwallet.utils import is_entropy
    >>> is_entropy(entropy="ee535b143b0d9d1f87546f9df0d06b1a")
    True
    """

    try:
        return len(unhexlify(entropy)) in [16, 20, 24, 28, 32]
    except:
        return False


def is_mnemonic(mnemonic: str, language: Optional[str] = None) -> bool:
    """
    Check mnemonic words.

    :param mnemonic: Mnemonic words.
    :type mnemonic: str
    :param language: Mnemonic language, default to None.
    :type language: str

    :returns: bool -- Mnemonic valid/invalid.

    >>> from hdwallet.utils import is_mnemonic
    >>> is_mnemonic(mnemonic="sceptre capter séquence girafe absolu relatif fleur zoologie muscle sirop saboter parure")
    True
    """

    if language and language not in ["english", "french", "italian", "japanese",
                                     "chinese_simplified", "chinese_traditional", "korean", "spanish"]:
        raise ValueError("invalid language, use only this options english, french, "
                         "italian, spanish, chinese_simplified, chinese_traditional, japanese or korean languages.")
    try:
        mnemonic = unicodedata.normalize("NFKD", mnemonic)
        if language is None:
            for _language in ["english", "french", "italian",
                              "chinese_simplified", "chinese_traditional", "japanese", "korean", "spanish"]:
                valid = False
                if Mnemonic(language=_language).check(mnemonic=mnemonic) is True:
                    valid = True
                    break
            return valid
        else:
            return Mnemonic(language=language).check(mnemonic=mnemonic)
    except:
        return False


def get_entropy_strength(entropy: str) -> int:
    """
    Get entropy strength.

    :param entropy: Entropy hex string.
    :type entropy: str

    :returns: int -- Entropy strength.

    >>> from hdwallet.utils import get_entropy_strength
    >>> get_entropy_strength(entropy="ee535b143b0d9d1f87546f9df0d06b1a")
    128
    """

    if not is_entropy(entropy=entropy):
        raise ValueError("Invalid entropy hex string.")

    length = len(unhexlify(entropy))
    if length == 16:
        return 128
    elif length == 20:
        return 160
    elif length == 24:
        return 192
    elif length == 28:
        return 224
    elif length == 32:
        return 256


def get_mnemonic_strength(mnemonic: str, language: Optional[str] = None) -> int:
    """
    Get mnemonic strength.

    :param mnemonic: Mnemonic words.
    :type mnemonic: str
    :param language: Mnemonic language, default to None.
    :type language: str

    :returns: int -- Mnemonic strength.

    >>> from hdwallet.utils import get_mnemonic_strength
    >>> get_mnemonic_strength(mnemonic="sceptre capter séquence girafe absolu relatif fleur zoologie muscle sirop saboter parure")
    128
    """

    if not is_mnemonic(mnemonic=mnemonic, language=language):
        raise ValueError("Invalid mnemonic words.")

    words = len(unicodedata.normalize("NFKD", mnemonic).split(" "))
    if words == 12:
        return 128
    elif words == 15:
        return 160
    elif words == 18:
        return 192
    elif words == 21:
        return 224
    elif words == 24:
        return 256


def get_mnemonic_language(mnemonic: str) -> str:
    """
    Get mnemonic language.

    :param mnemonic: Mnemonic words.
    :type mnemonic: str

    :returns: str -- Mnemonic language.

    >>> from hdwallet.utils import get_mnemonic_language
    >>> get_mnemonic_language(mnemonic="sceptre capter séquence girafe absolu relatif fleur zoologie muscle sirop saboter parure")
    "french"
    """

    if not is_mnemonic(mnemonic=mnemonic):
        raise ValueError("Invalid mnemonic words.")

    language = None
    mnemonic = unicodedata.normalize("NFKD", mnemonic)
    for _language in ["english", "french", "italian",
                      "chinese_simplified", "chinese_traditional", "japanese", "korean", "spanish"]:
        if Mnemonic(language=_language).check(mnemonic=mnemonic) is True:
            language = _language
            break
    return language


def entropy_to_mnemonic(entropy: str, language: str = "english") -> str:
    """
    Get mnemonic from entropy hex string.

    :param entropy: Entropy hex string.
    :type entropy: str
    :param language: Mnemonic language, default to english.
    :type language: str

    :returns: str -- Mnemonic words.

    >>> from hdwallet.utils import entropy_to_mnemonic
    >>> entropy_to_mnemonic(entropy="ee535b143b0d9d1f87546f9df0d06b1a", language="korean")
    "학력 외침 주민 스위치 출연 연습 근본 여전히 울음 액수 귀신 마누라"
    """

    if not is_entropy(entropy=entropy):
        raise ValueError("Invalid entropy hex string.")

    if language and language not in ["english", "french", "italian", "japanese",
                                     "chinese_simplified", "chinese_traditional", "korean", "spanish"]:
        raise ValueError("Invalid language, use only this options english, french, "
                         "italian, spanish, chinese_simplified, chinese_traditional, japanese or korean languages.")

    return Mnemonic(language=language).to_mnemonic(unhexlify(entropy))


def mnemonic_to_entropy(mnemonic: str, language: Optional[str] = None) -> str:
    """
    Get entropy from mnemonic words.

    :param mnemonic: Mnemonic words.
    :type mnemonic: str
    :param language: Mnemonic language, default to english.
    :type language: str

    :returns: str -- Enropy hex string.

    >>> from hdwallet.utils import mnemonic_to_entropy
    >>> mnemonic_to_entropy(mnemonic="학력 외침 주민 스위치 출연 연습 근본 여전히 울음 액수 귀신 마누라", language="korean")
    "ee535b143b0d9d1f87546f9df0d06b1a"
    """

    if not is_mnemonic(mnemonic=mnemonic, language=language):
        raise ValueError("Invalid mnemonic words.")

    mnemonic = unicodedata.normalize("NFKD", mnemonic)
    language = language if language else get_mnemonic_language(mnemonic=mnemonic)
    return Mnemonic(language=language).to_entropy(mnemonic).hex()


def is_root_xprivate_key(xprivate_key: str, symbol: str) -> bool:
    decoded_xprivate_key = check_decode(xprivate_key)
    if len(decoded_xprivate_key) != 78:  # 78, 156
        raise ValueError("Invalid xprivate key.")
    cryptocurrency = get_cryptocurrency(symbol=symbol)
    semantic = get_semantic(_cryptocurrency=cryptocurrency, version=decoded_xprivate_key[:4], key_type="private_key")
    version = cryptocurrency.EXTENDED_PRIVATE_KEY.__getattribute__(
        semantic.upper()
    )
    if version is None:
        raise NotImplementedError(semantic)
    raw = f"{_unhexlify(version).hex()}000000000000000000"
    return decoded_xprivate_key.hex().startswith(raw)


def is_root_xpublic_key(xpublic_key: str, symbol: str) -> bool:
    decoded_xpublic_key = check_decode(xpublic_key)
    if len(decoded_xpublic_key) != 78:  # 78, 156
        raise ValueError("Invalid xpublic key.")
    cryptocurrency = get_cryptocurrency(symbol=symbol)
    semantic = get_semantic(_cryptocurrency=cryptocurrency, version=decoded_xpublic_key[:4], key_type="public_key")
    version = cryptocurrency.EXTENDED_PUBLIC_KEY.__getattribute__(
        semantic.upper()
    )
    if version is None:
        raise NotImplementedError(semantic)
    raw = f"{_unhexlify(version).hex()}000000000000000000"
    return decoded_xpublic_key.hex().startswith(raw)



#!/usr/bin/env python3


from hdwallet.cli.__main__ import main as cli_main


def test_strengths(cli_tester):

    hdwallet = cli_tester.invoke(
        cli_main, [
            "list", "strengths"
        ]
    )

    assert hdwallet.exit_code == 0
    assert hdwallet.output

    hdwallet = cli_tester.invoke(
        cli_main, [
            "l", "strengths"
        ]
    )

    assert hdwallet.exit_code == 0
    assert hdwallet.output

    hdwallet = cli_tester.invoke(
        cli_main, [
            "list", "s"
        ]
    )

    assert hdwallet.exit_code == 0
    assert hdwallet.output

    hdwallet = cli_tester.invoke(
        cli_main, [
            "l", "s"
        ]
    )

    assert hdwallet.exit_code == 0
    assert hdwallet.output



#!/usr/bin/env python3

from hashlib import sha256

import sha3
import six


__base58_alphabet = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
__base58_alphabet_bytes = b"123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
__base58_radix = len(__base58_alphabet)


def checksum_encode(address, crypto="eth"):
    out = ""
    keccak = sha3.keccak_256()
    addr = address.lower().replace("0x", "") if crypto == "eth" else address.lower().replace("xdc", "")
    keccak.update(addr.encode("ascii"))
    hash_addr = keccak.hexdigest()
    for i, c in enumerate(addr):
        if int(hash_addr[i], 16) >= 8:
            out += c.upper()
        else:
            out += c
    return ("0x" + out) if crypto == "eth" else ("xdc" + out)


def string_to_int(data):
    val = 0

    if type(data) == str:
        data = bytearray(data)

    for (i, c) in enumerate(data[::-1]):
        val += (256 ** i) * c
    return val


def ensure_string(data):
    if isinstance(data, six.binary_type):
        return data.decode("utf-8")
    elif not isinstance(data, six.string_types):
        raise ValueError("Invalid value for string")
    return data


def encode(data):
    enc = ""
    val = string_to_int(data)
    while val >= __base58_radix:
        val, mod = divmod(val, __base58_radix)
        enc = __base58_alphabet[mod] + enc
    if val:
        enc = __base58_alphabet[val] + enc

    n = len(data) - len(data.lstrip(b"\0"))
    return __base58_alphabet[0] * n + enc


def check_encode(raw):
    chk = sha256(sha256(raw).digest()).digest()[:4]
    return encode(raw + chk)


def decode(data):
    if bytes != str:
        data = bytes(data, "ascii")

    val = 0
    prefix = 0
    for c in data:
        val = (val * __base58_radix) + __base58_alphabet_bytes.find(c)
        if val == 0:
            prefix += 1

    dec = bytearray()
    while val > 0:
        val, mod = divmod(val, 256)
        dec.append(mod)

    dec.extend(bytearray(prefix))

    return bytes(dec[::-1])


def check_decode(enc):
    dec = decode(enc)
    raw, chk = dec[:-4], dec[-4:]
    if chk != sha256(sha256(raw).digest()).digest()[:4]:
        raise ValueError("base58 decoding checksum error")
    else:
        return raw



#!/usr/bin/env python3

from .hdwallet import (
    HDWallet,
    BIP32HDWallet,
    BIP44HDWallet,
    BIP49HDWallet,
    BIP84HDWallet,
    BIP141HDWallet
)

# HDWallet Information's
__version__: str = "v2.1.1"
__license__: str = "ISCL"
__author__: str = "Meheret Tesfaye Batu"
__email__: str = "meherett@zoho.com"
__description__: str = "Python-based library for the implementation of a hierarchical deterministic wallet " \
                       "generator for more than 140+ multiple cryptocurrencies."

__all__: list = [
    "__version__",
    "__license__",
    "__author__",
    "__email__",
    "__description__",
    "HDWallet",
    "BIP32HDWallet",
    "BIP44HDWallet",
    "BIP49HDWallet",
    "BIP84HDWallet",
    "BIP141HDWallet"
]
